-- Power bar module

local maxpower = 0
local currentpower = 0
local powertype
local powcolor = {}

function SN:SetupPower()

	local f = SN.F
	local v = SN.Media
	f.powercontainer = CreateFrame("Frame", nil, UIParent)
	f.powercontainer:SetPoint("CENTER", UIParent, "CENTER", 0, 100)
	f.powercontainer:SetSize(250, 22)
	f.powercontainer:SetMovable(true)
	f.powercontainer:EnableMouse(true)
	f.powercontainer:SetScript("OnMouseDown", function(self, button)
		if button == "LeftButton" and not self.isMoving then
			self:StartMoving()
			self.isMoving = true
		end
	end)
	f.powercontainer:SetScript("OnMouseUp", function(self, button)
		if button == "LeftButton" and self.isMoving then
			self:StopMovingOrSizing()
			self.isMoving = false
		end
	end)
	--The actual power bar
	f.power = CreateFrame("StatusBar", nil, f.powercontainer)
	f.power:SetPoint("CENTER", f.powercontainer, "CENTER", 0, 0)
	f.power:SetSize(250, 22)
	f.power:SetStatusBarTexture(v.media.minTex)	
	f.power:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
	f.power:SetMinMaxValues(0, (UnitPowerMax("player") > 0) and UnitPowerMax("player") or 100)	
	f.power:SetValue(UnitPower("player"))
	f.power.TimeSinceLastUpdate = 0
	f.power:SetScript("OnUpdate", UpdatePower)
	f.power:SetBackdrop({
				bgFile = v.media.blankTex,
			})
	f.power:SetBackdropColor(0.1, 0.1, 0.1, 0)
	--The power text
	f.power.value = f.power:CreateFontString(nil, "OVERLAY")
	f.power.value:SetPoint("CENTER", f.power, "CENTER", 4, 0)
	f.power.value:SetFont(v.media.font, 14, "OUTLINE")
	f.power.value:SetJustifyH("CENTER")
	f.power.value:SetShadowOffset(1, -1)
	f.power.value:SetTextColor(0, 1, 0)
	f.power.value:SetText(currentpower.." / "..maxpower)
	--SN:MakeSmooth(f.power)
	SN:SNCreateBG(f.powercontainer)
	-- Register Events to support the bar
	f.power:RegisterEvent("UNIT_SPELLCAST_START")
	f.power:RegisterEvent("UNIT_SPELLCAST_STOP")
	f.power:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	f.power:RegisterEvent("UNIT_SPELLCAST_FAILED")
	f.power:RegisterEvent("UNIT_MAXPOWER")
	f.power:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
end

function SN:GetPowerColor()
	
	powertype = select(2, UnitPowerType("player"))
	if powertype == "MANA" then
		powcolor = SN.Media.power.MANA
	elseif powertype == "RAGE" then
		powcolor = SN.Media.power.RAGE
	elseif powertype == "FOCUS" then
		powcolor = SN.Media.power.FOCUS
	elseif powertype == "ENERGY" then
		powcolor = SN.Media.power.ENERGY
	elseif powertype == "RUNIC_POWER" then
		powcolor = SN.Media.power.RUNIC_POWER
	end
	return powcolor
	
end
function UpdatePower(frame, elapsed)
	frame.TimeSinceLastUpdate = frame.TimeSinceLastUpdate + elapsed
	if frame.TimeSinceLastUpdate > SN_UpdateInterval then
		frame:SetValue(UnitPower("player"))
		currentpower = UnitPower("player")
		maxpower = UnitPower("player")
		frame:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
		frame.value:SetText(currentpower.." / "..maxpower)
		frame.TimeSinceLastUpdate = 0
	end
end

function SN:UpdatePowerType()
	SN:GetPowerColor()
	SN.F.power:SetValue(UnitPower("player"))
	currentpower = UnitPower("player")
	maxpower = UnitPower("player")
	SN.F.power:SetStatusBarColor(powcolor.r, powcolor.g, powcolor.b, 1)
	SN.F.power.value:SetText(currentpower.." / "..maxpower)
end

local function Smooth(self, value)
	if (value ~= self:GetValue()) or (value == 0) then
		self.smoothing = value
	else
		self.smoothing = nil
	end
end

function SN:MakeSmooth(powerframe)
	if powerframe.SetValue_ORI then return end
	powerframe.SetValue_ORI = powerframe.SetValue
	powerframe.SetValue = Smooth
	
	powerframe.smoother = CreateFrame("Frame", nil, powerframe)
	local min, max = math.min, math.max
	powerframe.smoother:SetScript("OnUpdate", function(self)
		local rate = GetFramerate()
		local limit = 30 / rate
		if self:GetParent().smoothing then
			local cur = self:GetParent():GetValue()
			local new = cur + min((self:GetParent().smoothing - cur) / 3, max(self:GetParent().smoothing - cur, limit))
			self:GetParent():SetValue_ORI(new)
			if cur == self:GetParent().smoothing or abs(new - self:GetParent().smoothing) < 2 then
				self:GetParent():SetValue_ORI(self:GetParent().smoothing)
				self:GetParent().smoothing = nil
			end
		end
	end)
end

function SN:RemoveSmooth(powerframe)
	if not powerframe.SetValue_ORI then return end
	
	powerframe.smoother:SetScript("OnUpdate", nil)
	powerframe.smoother:SetParent(nil)
	powerframe.smoother = nil
	
	powerframe.SetValue = powerframe.SetValue_ORI
	powerframe.SetValue_ORI = nil
end

function SN:PowebarTicks()

	local name, cost
	for i=1,5 do
		if (SN.V.profile.powerbar.ticks[i][1] == true) and ( (SN.V.profile.powerbar.ticks[i][6] == 0)
			or (SN.V.profile.powerbar.ticks[i][6] == GetSpecialization() ) ) and (SN.V.profile.powerbar.ticks[1][1] == true) then
				
			local _, _name, _cost
			if ii > 1 then
				name, _, _, cost = GetSpellInfo(SN.V.profile.powerbar.ticks[i][2])
			end
				
			if (ii == 1) or ( (ii > 1) and name and cost and (cost > 0) ) then
				SN.F.power["indicatorTick"..i] = CreateFrame("Frame", "SN_POWER_TICKS"..i, SN.F.power)
				SN.F.power["indicatorTick"..i]:SetSize(10, SN.F.power:GetHeight() * 1.6)
				SN.F.power["indicatorTick"..i].tex = SN.F.power["indicatorTick"..i]:CreateTexture(nil, "OVERLAY")
				SN.F.power["indicatorTick"..i].tex:SetAllPoints(SN.F.power["indicatorTick"..i])
				SN.F.power["indicatorTick"..i].tex:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")	
				SN.F.power["indicatorTick"..i].tex:SetBlendMode("ADD")
				SN.F.power["indicatorTick"..i]:SetAlpha(1)
				SN.F.power["indicatorTick"..i]:Hide()
			end
		end
	end
end